from django import http
from django.conf.urls import url
from django.core.exceptions import ImproperlyConfigured
from django.shortcuts import get_object_or_404
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

import requests
from mooch.base import BaseMoocher, csrf_exempt_m, require_POST_m
from mooch.signals import post_charge


class StripeMoocher(BaseMoocher):
    identifier = "stripe"
    title = _("Pay with Stripe")

    def __init__(self, *, publishable_key, secret_key, **kwargs):
        if any(x is None for x in (publishable_key, secret_key)):
            raise ImproperlyConfigured(
                "%s: None is not allowed in (%r, %r)"
                % (self.__class__.__name__, publishable_key, secret_key)
            )

        self.publishable_key = publishable_key
        self.secret_key = secret_key = secret_key
        super().__init__(**kwargs)

    def get_urls(self):
        return [url(r"^stripe_charge/$", self.charge_view, name="stripe_charge")]

    def payment_form(self, request, payment):
        return render_to_string(
            "mooch/stripe_payment_form.html",
            {
                "moocher": self,
                "payment": payment,
                "publishable_key": self.publishable_key,
                "charge_url": reverse("%s:stripe_charge" % self.app_name),
                "LANGUAGE_CODE": getattr(request, "LANGUAGE_CODE", "auto"),
            },
            request=request,
        )

    @csrf_exempt_m
    @require_POST_m
    def charge_view(self, request):
        instance = get_object_or_404(self.model, id=request.POST.get("id"))
        instance.payment_service_provider = self.identifier
        instance.transaction = repr(
            {key: values for key, values in request.POST.lists() if key != "token"}
        )
        instance.save()

        response = requests.post(
            "https://api.stripe.com/v1/charges",
            auth=(self.secret_key, ""),
            data={
                "amount": instance.amount_cents,
                "source": request.POST.get("token"),
                "currency": "CHF",
            },
            headers={"Idempotency-Key": instance.id.hex},
            timeout=5,
        )

        instance.charged_at = timezone.now()  # TODO handle 402 (card reject)
        instance.transaction = response.text
        instance.save()

        post_charge.send(sender=self, payment=instance, request=request)

        return http.HttpResponse("OK")



from django.utils.decorators import method_decorator
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST


csrf_exempt_m = method_decorator(csrf_exempt)
require_POST_m = method_decorator(require_POST)


class BaseMoocher(object):
    #: The model instance used for mooching
    model = None

    identifier = None
    title = _("Pay")

    success_url = "/"
    failure_url = "/"

    app_name = "mooch"

    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    @property
    def urls(self):
        return self.get_urls()

    def get_urls(self):
        return []

    def payment_form(self):
        pass



from django.core.mail import EmailMultiAlternatives
from django.template.loader import TemplateDoesNotExist, render_to_string


def render_to_mail(template, context, **kwargs):
    """
    Renders a mail and returns the resulting ``EmailMultiAlternatives``
    instance

    * ``template``: The base name of the text and HTML (optional) version of
      the mail.
    * ``context``: The context used to render the mail. This context instance
      should contain everything required.
    * Additional keyword arguments are passed to the ``EmailMultiAlternatives``
      instantiation. Use those to specify the ``to``, ``headers`` etc.
      arguments.

    Usage example::

        # Render the template myproject/hello_mail.txt (first non-empty line
        # contains the subject, third to last the body) and optionally the
        # template myproject/hello_mail.html containing the alternative HTML
        # representation.
        message = render_to_mail('myproject/hello_mail', {}, to=[email])
        message.send()
    """
    lines = iter(render_to_string("%s.txt" % template, context).splitlines())

    subject = ""
    while True:
        line = next(lines)
        if line:
            subject = line
            break

    body = "\n".join(lines).strip("\n")
    message = EmailMultiAlternatives(subject=subject, body=body, **kwargs)

    try:
        message.attach_alternative(
            render_to_string("%s.html" % template, context), "text/html"
        )
    except TemplateDoesNotExist:
        pass

    return message



import locale
import logging
from hashlib import sha1

from django import http
from django.conf.urls import url
from django.contrib import messages
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.core.mail import mail_managers
from django.shortcuts import redirect
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.utils.translation import get_language, to_locale, ugettext_lazy as _

from mooch.base import BaseMoocher, csrf_exempt_m, require_POST_m
from mooch.signals import post_charge


logger = logging.getLogger("mooch.postfinance")


class PostFinanceMoocher(BaseMoocher):
    identifier = "postfinance"
    title = _("Pay with PostFinance")

    def __init__(
        self, *, pspid, live, sha1_in, sha1_out, payment_methods=None, **kwargs
    ):
        if any(x is None for x in (pspid, live, sha1_in, sha1_out)):
            raise ImproperlyConfigured(
                "%s: None is not allowed in (%r, %r, %r, %r)"
                % (self.__class__.__name__, pspid, live, sha1_in, sha1_out)
            )

        self.pspid = pspid
        # Which payment options should be shown
        # Options: PostFinance Card, PostFinance e-finance, TWINT, PAYPAL
        self.payment_methods = (
            ["PostFinance Card", "PostFinance e-finance"]
            if payment_methods is None
            else payment_methods
        )
        self.live = live
        self.sha1_in = sha1_in
        self.sha1_out = sha1_out
        super().__init__(**kwargs)

    def get_urls(self):
        return [
            url(
                r"^postfinance_success/$", self.success_view, name="postfinance_success"
            ),
            url(
                r"^postfinance_postsale/$",
                self.postsale_view,
                name="postfinance_postsale",
            ),
        ]

    def payment_form(self, request, payment):
        postfinance = {
            # Add a random suffix, because PostFinance does not like
            # processing the same order ID over and over.
            "orderID": "%s-%s" % (payment.id.hex, get_random_string(4)),
            "amount": str(payment.amount_cents),
            "currency": "CHF",
            "PSPID": self.pspid,
            "language": locale.normalize(to_locale(get_language())).split(".")[0],
            "EMAIL": payment.email,
        }

        postfinance["SHASign"] = sha1(
            (
                "".join(
                    (
                        postfinance["orderID"],
                        postfinance["amount"],
                        postfinance["currency"],
                        postfinance["PSPID"],
                        self.sha1_in,
                    )
                )
            ).encode("utf-8")
        ).hexdigest()

        return render_to_string(
            "mooch/postfinance_payment_form.html",
            {
                "moocher": self,
                "payment": payment,
                "postfinance": postfinance,
                "mode": "prod" if self.live else "test",
                "payment_methods": self.payment_methods,
                "success_url": request.build_absolute_uri(
                    reverse("%s:postfinance_success" % self.app_name)
                ),
                "failure_url": request.build_absolute_uri(str(self.failure_url)),
            },
            request=request,
        )

    def _process_query(self, data, request):
        try:
            parameters_repr = repr(data).encode("utf-8")
            logger.info("IPN: Processing request data %s" % parameters_repr)

            try:
                orderID = data["orderID"]
                currency = data["currency"]
                amount = data["amount"]
                PM = data["PM"]
                ACCEPTANCE = data["ACCEPTANCE"]
                STATUS = data["STATUS"]
                CARDNO = data["CARDNO"]
                PAYID = data["PAYID"]
                NCERROR = data["NCERROR"]
                BRAND = data["BRAND"]
                SHASIGN = data["SHASIGN"]
            except KeyError:
                logger.error("IPN: Missing data in %s" % parameters_repr)
                raise ValidationError("Missing data")

            sha1_source = "".join(
                (
                    orderID,
                    currency,
                    amount,
                    PM,
                    ACCEPTANCE,
                    STATUS,
                    CARDNO,
                    PAYID,
                    NCERROR,
                    BRAND,
                    self.sha1_out,
                )
            )

            sha1_out = sha1(sha1_source.encode("utf-8")).hexdigest()

            if sha1_out.lower() != SHASIGN.lower():
                logger.error("IPN: Invalid hash in %s" % parameters_repr)
                raise ValidationError("Hash did not validate")

            try:
                instance = self.model.objects.get(pk=orderID.split("-")[0])
            except self.model.DoesNotExist:
                logger.error("IPN: Instance %s does not exist" % orderID)
                raise ValidationError("Instance %s does not exist" % orderID)

            if STATUS in ("5", "9"):
                instance.charged_at = timezone.now()

            instance.payment_service_provider = self.identifier
            instance.transaction = parameters_repr
            instance.save()

            post_charge.send(sender=self, payment=instance, request=request)

        except Exception as e:
            logger.error("IPN: Processing failure %s" % e)
            raise

    def success_view(self, request):
        try:
            self._process_query(request.GET.copy(), request)
        except ValidationError as exc:
            mail_managers(
                "Validation error in PostFinance success view",
                "\n".join(
                    [request.build_absolute_uri(), ""] + [m for m in exc.messages]
                ),
            )
            for m in exc.messages:
                messages.error(request, m)
            return redirect(self.failure_url)
        else:
            return redirect(self.success_url)

    @csrf_exempt_m
    @require_POST_m
    def postsale_view(self, request):
        try:
            self._process_query(request.POST.copy(), request)
        except ValidationError as exc:
            return http.HttpResponseForbidden(exc.message)

        return http.HttpResponse("OK")



from django import http
from django.conf.urls import url
from django.shortcuts import get_object_or_404
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from mooch.base import BaseMoocher, require_POST_m
from mooch.signals import post_charge


class BankTransferMoocher(BaseMoocher):
    identifier = "banktransfer"
    title = _("Pay by bank transfer")

    def __init__(self, *, autocharge, **kw):
        self.autocharge = autocharge
        super().__init__(**kw)

    def get_urls(self):
        return [
            url(
                "^banktransfer_confirm/$",
                self.confirm_view,
                name="banktransfer_confirm",
            )
        ]

    def payment_form(self, request, payment):
        return render_to_string(
            "mooch/banktransfer_payment_form.html",
            {
                "payment": payment,
                "moocher": self,
                "confirm_url": reverse("%s:banktransfer_confirm" % self.app_name),
            },
            request=request,
        )

    @require_POST_m
    def confirm_view(self, request):
        instance = get_object_or_404(self.model, id=request.POST.get("id"))
        instance.payment_service_provider = self.identifier
        if self.autocharge:
            instance.charged_at = timezone.now()
        instance.save()

        post_charge.send(sender=self, payment=instance, request=request)

        return http.HttpResponseRedirect(self.success_url)



#!/usr/bin/env python

import io
import os
from setuptools import setup, find_packages


def read(filename):
    with io.open(
        os.path.join(os.path.dirname(__file__), filename), encoding="utf-8"
    ) as f:
        return f.read()


setup(
    name="django-mooch",
    version=__import__("mooch").__version__,
    description="Simple payment for Django",
    long_description=read("README.rst"),
    author="Matthias Kestenholz",
    author_email="mk@feinheit.ch",
    url="https://github.com/matthiask/django-mooch/",
    license="MIT License",
    platforms=["OS Independent"],
    packages=find_packages(),
    include_package_data=True,
    install_requires=["requests>2"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
    ],
    zip_safe=False,
)



import uuid

from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _


class Payment(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    created_at = models.DateTimeField(_("created at"), default=timezone.now)
    charged_at = models.DateTimeField(_("charged at"), blank=True, null=True)
    amount = models.DecimalField(_("amount"), max_digits=10, decimal_places=2)
    payment_service_provider = models.CharField(
        _("payment service provider"), max_length=100, blank=True
    )
    email = models.EmailField(_("email"), max_length=254)
    transaction = models.TextField(_("transaction"), blank=True)

    class Meta:
        abstract = True
        ordering = ("-created_at",)
        verbose_name = _("payment")
        verbose_name_plural = _("payments")

    def __str__(self):
        return "%.2f" % self.amount

    @property
    def amount_cents(self):
        return int(self.amount * 100)
